local _, AT = ...
local DEBUG = false

Enums = {}
AT.Enums = Enums

local metatable = {
  __index = function()
    if DEBUG then
      error('Enum value does not exist.')
    end
  end
}

function Enums:HasFlag(aVal, ...)
  local all = bit.bor(...)
  return bit.band(aVal, all) == all
end

Enums.SCAN_RETURN_VALUE = {
  INVALID_UNIT = 0,
  SPELLS_FOUND = 1,
  NO_SPELLS_FOUND = 2
}
setmetatable(Enums.SCAN_RETURN_VALUE, metatable)

Enums.SCAN_RESULT = {
  UNIT = 0,
  UNIT_GUID = 1,
  CATEGORY = 2,
  BUFFS_INFO = 3,
  HAS_IMPORTANT_BUFFS = 4,
  HAS_MOUSEOVER_BUFFS = 5,
  HAS_OVERVIEW_BUFFS = 6,
  HAS_NAMEPLATE_BUFFS = 7
}
setmetatable(Enums.SCAN_RESULT, metatable)

Enums.BUFF_INFO = {
  ID = 0,
  NAME = 1,
  ICON = 2,
  COUNT = 3,
  EXPIRATION_TIME = 4,
  SHOW_IN_MOUSEOVER = 5,
  SHOW_IN_OVERVIEW = 6,
  SHOW_IN_NAMEPLATE = 7
}
setmetatable(Enums.BUFF_INFO, metatable)

Enums.SCAN_CATEGORY = {
  TARGET = 0,
  FOCUS = 1,
  PET_TARGET = 2,
  MOUSEOVER = 3,
  BOSS = 4,
  NAMEPLATE = 5,
  GROUP_MEMBER_TARGET = 6,
  GROUP_MEMBER_PET_TARGET = 7,
  ToStr = function(self, aVal)
    if not rawget(self, 'nameMap') then
      local t = {}
      t[self.TARGET] = "Target"
      t[self.FOCUS] = "Focus"
      t[self.PET_TARGET] = "Pet target"
      t[self.MOUSEOVER] = "Mouseover"
      t[self.BOSS] = "Boss"
      t[self.NAMEPLATE] = "Nameplate"
      t[self.GROUP_MEMBER_TARGET] = "Group member target"
      t[self.GROUP_MEMBER_PET_TARGET] = "Group member pet target"
      self.nameMap = t
    end
    return self.nameMap[aVal]
  end
}
setmetatable(Enums.SCAN_CATEGORY, metatable)

Enums.BUFF_STOLEN_CHAT_MESSAGE = {
  NONE = 0,
  PRIVATE = 1,
  PUBLIC = 2
}
setmetatable(Enums.BUFF_STOLEN_CHAT_MESSAGE, metatable)

Enums.NP_ORIENTATION = {
  LEFT = 0,
  CENTER = 1,
  RIGHT = 2
}
setmetatable(Enums.NP_ORIENTATION, metatable)

Enums.TRANSPARENCY_STATE = {
  NORMAL = 0,
  NO_MANA = 1,
  NOT_IN_RANGE = 2
}
setmetatable(Enums.TRANSPARENCY_STATE, metatable)

Enums.PROFILE_TYPE = {
  GENERAL = 1,
  FILTER = 2
}
setmetatable(Enums.PROFILE_TYPE, metatable)
