local _, AT = ...
local DEBUG = false
local function _assert(aCnd, aMsg) if DEBUG then assert(aCnd, aMsg) end end

local Constants = AT.Constants

local Utils = {}
AT.Utils = Utils

function Utils:Print(aMsg)
  print(string.format("|cff4fd0ff%s:|r %s", Constants.ADDON_DISPLAY_NAME, aMsg))
end

function Utils:GetResolution()
  local x, y = strsplit('x', Display_DisplayModeDropDown:windowedmode() and GetCVar('gxWindowedResolution') or GetCVar('gxFullscreenResolution'))
  return tonumber(x), tonumber(y)
end

function Utils:FormatExpirationTime(aExpirationTime)
  if aExpirationTime <= 0 then
    return '0s';
  end

  if aExpirationTime < 60 then
    return aExpirationTime..'s'
  end

  if aExpirationTime < 3600 then
    return math.floor(aExpirationTime / 60)..'m'
  end

  return math.floor(aExpirationTime / 3600)..'h'
end

function Utils:TableShallowCopy(aTable)
  local copy = {}
  for k, v in pairs(aTable) do
      copy[k] = v
  end
  return copy
end

function Utils:TableDeepCopy(aTable)
  local copy = {}
  for k, v in pairs(aTable) do
      if type(orig) == 'table' then
        copy[k] = self:TableDeepCopy(v)
      else
        copy[k] = v
      end
  end
  return copy
end

function Utils:TableLen(aTable)
  local len = 0
  for _, _ in pairs(aTable) do
    len = len + 1
  end
  return len
end

function Utils:TableValueIndex(aTable, aVal)
  for i, v in ipairs(aTable) do
    if aVal == v then
      return i
    end
  end
end

function Utils:TableConcat(aTable, ...)
  local t = {}
  for _, v in pairs(aTable) do
    tinsert(t, v)
  end
  return table.concat(t, ...)
end

function Utils:StrIsNilOrEmpty(aStr)
  return aStr == nil or aStr == ''
end

function Utils:StrIsNilOrEmptyOrWhitespaceOnly(aStr)
  return Utils:StrIsNilOrEmpty(aStr) or strtrim(aStr) == ''
end

function Utils:StrStartsWith(aStr1, aStr2)
  return aStr1:sub(1, aStr2:len()) == aStr2
end

function Utils:StrFirstCharToUpper(aStr)
  return aStr:gsub("^%l", strupper)
end

function Utils:Round(aNumber, aDecimals)
  local mult = 10^(aDecimals or 0)
  return math.floor(aNumber * mult + 0.5) / mult
end

function Utils:Clamp(aLow, aNumber, aHigh)
  return math.min(math.max(aNumber, aLow), aHigh)
end

function Utils:Ternary(aCond, aTrue, aFalse)
  if (aCond) then
    return aTrue
  end
  return aFalse
end

function Utils:RgbToHexStr(aColor)
  local result = 'ff'
  local colors = { aColor.r, aColor.g, aColor.b }
  for _, v in ipairs(colors) do
    local str = string.format('%x', v * 255)
    if str:len() == 1 then
      str = '0'..str
    end
    _assert(str:len() == 2)
    result = result..str
  end
  return result
end
